# generating a simulated dataset of multi-dimension events
# July 2020

n<-500 #length of time series
m<-100 #dimension of time series

# generate normal data (without event)
set.seed(123)
mu<-runif(m, min=-10, max=10)
sd<-1
ts_normal<-matrix(0, nrow=n, ncol=m)
for(i in 1:m){
  ts_normal[,i]<-rnorm(n, mean=mu[i], sd=sd)
}

# generate events
o<-1000 #number of events
min_l<-5 #minimum event length
max_l<-20 #maximum event length
min_d<-3 #minimum event dimension
max_d<-7 #maximum event dimension
events<-list()
set.seed(123)
for(i in 1:o){
  begin<-sample(1:(n-max_l), 1)
  e_length<-sample(min_l:max_l, 1)
  e_dim_size<-sample(min_d:max_d, 1)
  e_dim<-sample(1:m, e_dim_size)
  e_prop<-sample(c(-1, 1), e_dim_size, replace=T)
  events[[i]]<-list(c(begin, begin+e_length), e_dim, e_prop)
}

# add events to normal data
ts_sim<-ts_normal
for(i in 1:o){
  begin<-events[[i]][[1]][1]
  end<-events[[i]][[1]][[2]]
  e_dims<-events[[i]][[2]]
  e_props<-events[[i]][[3]]
  for(j in 1:length(e_dims)){
    cd<-e_dims[j]
    cp<-e_props[j]
    ts_sim[begin:end, cd]<-rnorm((end-begin+1), mean=mu[cd]+cp*2*sd, sd=sd)
  }
}

write.csv(ts_sim, "simulated_o=1000.csv", quote=F, row.names=F)

saveRDS(events, file = "E_sim_o=1000.RDS")
